# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import RoomSession
from telnyx._utils import parse_date
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.rooms import (
    SessionRetrieveResponse,
)
from telnyx.types.shared import RoomParticipant

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        session = client.rooms.sessions.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        session = client.rooms.sessions.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            include_participants=True,
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.rooms.sessions.with_raw_response.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.rooms.sessions.with_streaming_response.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionRetrieveResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_session_id` but received ''"):
            client.rooms.sessions.with_raw_response.retrieve(
                room_session_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_0(self, client: Telnyx) -> None:
        session = client.rooms.sessions.list_0()
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_0_with_all_params(self, client: Telnyx) -> None:
        session = client.rooms.sessions.list_0(
            filter={
                "active": True,
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            },
            include_participants=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_0(self, client: Telnyx) -> None:
        response = client.rooms.sessions.with_raw_response.list_0()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_0(self, client: Telnyx) -> None:
        with client.rooms.sessions.with_streaming_response.list_0() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_1(self, client: Telnyx) -> None:
        session = client.rooms.sessions.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_1_with_all_params(self, client: Telnyx) -> None:
        session = client.rooms.sessions.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            filter={
                "active": True,
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
            },
            include_participants=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_1(self, client: Telnyx) -> None:
        response = client.rooms.sessions.with_raw_response.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_1(self, client: Telnyx) -> None:
        with client.rooms.sessions.with_streaming_response.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[RoomSession], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_1(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            client.rooms.sessions.with_raw_response.list_1(
                room_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_participants(self, client: Telnyx) -> None:
        session = client.rooms.sessions.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_participants_with_all_params(self, client: Telnyx) -> None:
        session = client.rooms.sessions.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            filter={
                "context": "Alice",
                "date_joined_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_left_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_participants(self, client: Telnyx) -> None:
        response = client.rooms.sessions.with_raw_response.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_participants(self, client: Telnyx) -> None:
        with client.rooms.sessions.with_streaming_response.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve_participants(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_session_id` but received ''"):
            client.rooms.sessions.with_raw_response.retrieve_participants(
                room_session_id="",
            )


class TestAsyncSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            include_participants=True,
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.sessions.with_raw_response.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.sessions.with_streaming_response.retrieve(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionRetrieveResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_session_id` but received ''"):
            await async_client.rooms.sessions.with_raw_response.retrieve(
                room_session_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_0(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.list_0()
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_0_with_all_params(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.list_0(
            filter={
                "active": True,
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            },
            include_participants=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_0(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.sessions.with_raw_response.list_0()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_0(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.sessions.with_streaming_response.list_0() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_1(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_1_with_all_params(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            filter={
                "active": True,
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
            },
            include_participants=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_1(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.sessions.with_raw_response.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_1(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.sessions.with_streaming_response.list_1(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[RoomSession], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_1(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            await async_client.rooms.sessions.with_raw_response.list_1(
                room_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_participants_with_all_params(self, async_client: AsyncTelnyx) -> None:
        session = await async_client.rooms.sessions.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            filter={
                "context": "Alice",
                "date_joined_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_left_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.sessions.with_raw_response.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.sessions.with_streaming_response.retrieve_participants(
            room_session_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[RoomParticipant], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_session_id` but received ''"):
            await async_client.rooms.sessions.with_raw_response.retrieve_participants(
                room_session_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List

from ..._models import BaseModel
from .child_relationship_info import ChildRelationshipInfo
from .parent_relationship_info import ParentRelationshipInfo

__all__ = ["MetadataRetrieveRecordTypeResponse", "Meta"]


class Meta(BaseModel):
    max_recommended_depth: int

    total_children: int

    total_parents: int

    total_siblings: int


class MetadataRetrieveRecordTypeResponse(BaseModel):
    aliases: List[str]

    child_relationships: List[ChildRelationshipInfo]

    event: str

    examples: Dict[str, object]
    """Example queries and responses for this record type."""

    meta: Meta

    parent_relationships: List[ParentRelationshipInfo]

    product: str

    record_type: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .portout_comment import PortoutComment
from ..shared.metadata import Metadata

__all__ = ["CommentListResponse"]


class CommentListResponse(BaseModel):
    data: Optional[List[PortoutComment]] = None

    meta: Optional[Metadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["SupportingDocumentCreateParams", "Document"]


class SupportingDocumentCreateParams(TypedDict, total=False):
    documents: Iterable[Document]
    """List of supporting documents parameters"""


class Document(TypedDict, total=False):
    document_id: Required[str]
    """Identifies the associated document"""

    type: Required[Literal["loa", "invoice"]]
    """Identifies the type of the document"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RegisterCreateParams"]


class RegisterCreateParams(TypedDict, total=False):
    registration_codes: Required[SequenceNotStr[str]]

    sim_card_group_id: str
    """The group SIMCardGroup identification.

    This attribute can be <code>null</code> when it's present in an associated
    resource.
    """

    status: Literal["enabled", "disabled", "standby"]
    """Status on which the SIM card will be set after being successful registered."""

    tags: SequenceNotStr[str]
    """Searchable tags associated with the SIM card"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    voice_design_list_params,
    voice_design_create_params,
    voice_design_rename_params,
    voice_design_retrieve_params,
    voice_design_download_sample_params,
)
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.voice_design_list_response import VoiceDesignListResponse
from ..types.voice_design_create_response import VoiceDesignCreateResponse
from ..types.voice_design_rename_response import VoiceDesignRenameResponse
from ..types.voice_design_retrieve_response import VoiceDesignRetrieveResponse

__all__ = ["VoiceDesignsResource", "AsyncVoiceDesignsResource"]


class VoiceDesignsResource(SyncAPIResource):
    """Create and manage AI-generated voice designs using natural language prompts."""

    @cached_property
    def with_raw_response(self) -> VoiceDesignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoiceDesignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceDesignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoiceDesignsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        prompt: str,
        text: str,
        language: str | Omit = omit,
        max_new_tokens: int | Omit = omit,
        name: str | Omit = omit,
        repetition_penalty: float | Omit = omit,
        temperature: float | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        voice_design_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignCreateResponse:
        """Creates a new voice design (version 1) when `voice_design_id` is omitted.

        When
        `voice_design_id` is provided, adds a new version to the existing design
        instead. A design can have at most 50 versions.

        Args:
          prompt: Natural language description of the voice style, e.g. 'Speak in a warm, friendly
              tone with a slight British accent'.

          text: Sample text to synthesize for this voice design.

          language: Language for synthesis. Supported values: Auto, Chinese, English, Japanese,
              Korean, German, French, Russian, Portuguese, Spanish, Italian. Defaults to Auto.

          max_new_tokens: Maximum number of tokens to generate. Default: 2048.

          name: Name for the voice design. Required when creating a new design
              (`voice_design_id` is not provided); ignored when adding a version. Cannot be a
              UUID.

          repetition_penalty:
              Repetition penalty to reduce repeated patterns in generated audio. Default:
              1.05.

          temperature: Sampling temperature controlling randomness. Higher values produce more varied
              output. Default: 0.9.

          top_k: Top-k sampling parameter — limits the token vocabulary considered at each step.
              Default: 50.

          top_p: Top-p (nucleus) sampling parameter — cumulative probability cutoff for token
              selection. Default: 1.0.

          voice_design_id: ID of an existing voice design to add a new version to. When provided, a new
              version is created instead of a new design.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/voice_designs",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "text": text,
                    "language": language,
                    "max_new_tokens": max_new_tokens,
                    "name": name,
                    "repetition_penalty": repetition_penalty,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_p": top_p,
                    "voice_design_id": voice_design_id,
                },
                voice_design_create_params.VoiceDesignCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDesignCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignRetrieveResponse:
        """
        Returns the latest version of a voice design, or a specific version when
        `?version=N` is provided. The `id` parameter accepts either a UUID or the design
        name.

        Args:
          version: Specific version number to retrieve. Defaults to the latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/voice_designs/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"version": version}, voice_design_retrieve_params.VoiceDesignRetrieveParams),
            ),
            cast_to=VoiceDesignRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["name", "-name", "created_at", "-created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[VoiceDesignListResponse]:
        """
        Returns a paginated list of voice designs belonging to the authenticated
        account.

        Args:
          filter_name: Case-insensitive substring filter on the name field.

          page_number: Page number for pagination (1-based).

          page_size: Number of results per page.

          sort: Sort order. Prefix with `-` for descending. Defaults to `-created_at`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/voice_designs",
            page=SyncDefaultFlatPagination[VoiceDesignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_design_list_params.VoiceDesignListParams,
                ),
            ),
            model=VoiceDesignListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a voice design and all of its versions.

        This action cannot
        be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/voice_designs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete_version(
        self,
        version: int,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a specific version of a voice design.

        The version number
        must be a positive integer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/voice_designs/{id}/versions/{version}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def download_sample(
        self,
        id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """Downloads the WAV audio sample for the voice design.

        Returns the latest
        version's sample by default, or a specific version when `?version=N` is
        provided. The `id` parameter accepts either a UUID or the design name.

        Args:
          version: Specific version number to download the sample for. Defaults to the latest
              version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "audio/wav", **(extra_headers or {})}
        return self._get(
            f"/voice_designs/{id}/sample",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"version": version}, voice_design_download_sample_params.VoiceDesignDownloadSampleParams
                ),
            ),
            cast_to=BinaryAPIResponse,
        )

    def rename(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignRenameResponse:
        """Updates the name of a voice design.

        All versions retain their other properties.

        Args:
          name: New name for the voice design.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/voice_designs/{id}",
            body=maybe_transform({"name": name}, voice_design_rename_params.VoiceDesignRenameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDesignRenameResponse,
        )


class AsyncVoiceDesignsResource(AsyncAPIResource):
    """Create and manage AI-generated voice designs using natural language prompts."""

    @cached_property
    def with_raw_response(self) -> AsyncVoiceDesignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceDesignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceDesignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoiceDesignsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        prompt: str,
        text: str,
        language: str | Omit = omit,
        max_new_tokens: int | Omit = omit,
        name: str | Omit = omit,
        repetition_penalty: float | Omit = omit,
        temperature: float | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        voice_design_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignCreateResponse:
        """Creates a new voice design (version 1) when `voice_design_id` is omitted.

        When
        `voice_design_id` is provided, adds a new version to the existing design
        instead. A design can have at most 50 versions.

        Args:
          prompt: Natural language description of the voice style, e.g. 'Speak in a warm, friendly
              tone with a slight British accent'.

          text: Sample text to synthesize for this voice design.

          language: Language for synthesis. Supported values: Auto, Chinese, English, Japanese,
              Korean, German, French, Russian, Portuguese, Spanish, Italian. Defaults to Auto.

          max_new_tokens: Maximum number of tokens to generate. Default: 2048.

          name: Name for the voice design. Required when creating a new design
              (`voice_design_id` is not provided); ignored when adding a version. Cannot be a
              UUID.

          repetition_penalty:
              Repetition penalty to reduce repeated patterns in generated audio. Default:
              1.05.

          temperature: Sampling temperature controlling randomness. Higher values produce more varied
              output. Default: 0.9.

          top_k: Top-k sampling parameter — limits the token vocabulary considered at each step.
              Default: 50.

          top_p: Top-p (nucleus) sampling parameter — cumulative probability cutoff for token
              selection. Default: 1.0.

          voice_design_id: ID of an existing voice design to add a new version to. When provided, a new
              version is created instead of a new design.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/voice_designs",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "text": text,
                    "language": language,
                    "max_new_tokens": max_new_tokens,
                    "name": name,
                    "repetition_penalty": repetition_penalty,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_p": top_p,
                    "voice_design_id": voice_design_id,
                },
                voice_design_create_params.VoiceDesignCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDesignCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignRetrieveResponse:
        """
        Returns the latest version of a voice design, or a specific version when
        `?version=N` is provided. The `id` parameter accepts either a UUID or the design
        name.

        Args:
          version: Specific version number to retrieve. Defaults to the latest version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/voice_designs/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"version": version}, voice_design_retrieve_params.VoiceDesignRetrieveParams
                ),
            ),
            cast_to=VoiceDesignRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["name", "-name", "created_at", "-created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VoiceDesignListResponse, AsyncDefaultFlatPagination[VoiceDesignListResponse]]:
        """
        Returns a paginated list of voice designs belonging to the authenticated
        account.

        Args:
          filter_name: Case-insensitive substring filter on the name field.

          page_number: Page number for pagination (1-based).

          page_size: Number of results per page.

          sort: Sort order. Prefix with `-` for descending. Defaults to `-created_at`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/voice_designs",
            page=AsyncDefaultFlatPagination[VoiceDesignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_design_list_params.VoiceDesignListParams,
                ),
            ),
            model=VoiceDesignListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a voice design and all of its versions.

        This action cannot
        be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/voice_designs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete_version(
        self,
        version: int,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a specific version of a voice design.

        The version number
        must be a positive integer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/voice_designs/{id}/versions/{version}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def download_sample(
        self,
        id: str,
        *,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """Downloads the WAV audio sample for the voice design.

        Returns the latest
        version's sample by default, or a specific version when `?version=N` is
        provided. The `id` parameter accepts either a UUID or the design name.

        Args:
          version: Specific version number to download the sample for. Defaults to the latest
              version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "audio/wav", **(extra_headers or {})}
        return await self._get(
            f"/voice_designs/{id}/sample",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"version": version}, voice_design_download_sample_params.VoiceDesignDownloadSampleParams
                ),
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def rename(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDesignRenameResponse:
        """Updates the name of a voice design.

        All versions retain their other properties.

        Args:
          name: New name for the voice design.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/voice_designs/{id}",
            body=await async_maybe_transform({"name": name}, voice_design_rename_params.VoiceDesignRenameParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDesignRenameResponse,
        )


class VoiceDesignsResourceWithRawResponse:
    def __init__(self, voice_designs: VoiceDesignsResource) -> None:
        self._voice_designs = voice_designs

        self.create = to_raw_response_wrapper(
            voice_designs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            voice_designs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            voice_designs.list,
        )
        self.delete = to_raw_response_wrapper(
            voice_designs.delete,
        )
        self.delete_version = to_raw_response_wrapper(
            voice_designs.delete_version,
        )
        self.download_sample = to_custom_raw_response_wrapper(
            voice_designs.download_sample,
            BinaryAPIResponse,
        )
        self.rename = to_raw_response_wrapper(
            voice_designs.rename,
        )


class AsyncVoiceDesignsResourceWithRawResponse:
    def __init__(self, voice_designs: AsyncVoiceDesignsResource) -> None:
        self._voice_designs = voice_designs

        self.create = async_to_raw_response_wrapper(
            voice_designs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            voice_designs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            voice_designs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            voice_designs.delete,
        )
        self.delete_version = async_to_raw_response_wrapper(
            voice_designs.delete_version,
        )
        self.download_sample = async_to_custom_raw_response_wrapper(
            voice_designs.download_sample,
            AsyncBinaryAPIResponse,
        )
        self.rename = async_to_raw_response_wrapper(
            voice_designs.rename,
        )


class VoiceDesignsResourceWithStreamingResponse:
    def __init__(self, voice_designs: VoiceDesignsResource) -> None:
        self._voice_designs = voice_designs

        self.create = to_streamed_response_wrapper(
            voice_designs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            voice_designs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            voice_designs.list,
        )
        self.delete = to_streamed_response_wrapper(
            voice_designs.delete,
        )
        self.delete_version = to_streamed_response_wrapper(
            voice_designs.delete_version,
        )
        self.download_sample = to_custom_streamed_response_wrapper(
            voice_designs.download_sample,
            StreamedBinaryAPIResponse,
        )
        self.rename = to_streamed_response_wrapper(
            voice_designs.rename,
        )


class AsyncVoiceDesignsResourceWithStreamingResponse:
    def __init__(self, voice_designs: AsyncVoiceDesignsResource) -> None:
        self._voice_designs = voice_designs

        self.create = async_to_streamed_response_wrapper(
            voice_designs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            voice_designs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            voice_designs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            voice_designs.delete,
        )
        self.delete_version = async_to_streamed_response_wrapper(
            voice_designs.delete_version,
        )
        self.download_sample = async_to_custom_streamed_response_wrapper(
            voice_designs.download_sample,
            AsyncStreamedBinaryAPIResponse,
        )
        self.rename = async_to_streamed_response_wrapper(
            voice_designs.rename,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import (
    conference_list_params,
    conference_create_params,
    conference_retrieve_params,
    conference_list_participants_params,
    conference_update_participant_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.conference import Conference
from ...types.conference_create_response import ConferenceCreateResponse
from ...types.conference_retrieve_response import ConferenceRetrieveResponse
from ...types.conference_list_participants_response import ConferenceListParticipantsResponse
from ...types.conference_update_participant_response import ConferenceUpdateParticipantResponse
from ...types.conference_retrieve_participant_response import ConferenceRetrieveParticipantResponse

__all__ = ["ConferencesResource", "AsyncConferencesResource"]


class ConferencesResource(SyncAPIResource):
    """Conference command operations"""

    @cached_property
    def actions(self) -> ActionsResource:
        """Conference command operations"""
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ConferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ConferencesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        call_control_id: str,
        name: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        client_state: str | Omit = omit,
        comfort_noise: bool | Omit = omit,
        command_id: str | Omit = omit,
        duration_minutes: int | Omit = omit,
        hold_audio_url: str | Omit = omit,
        hold_media_name: str | Omit = omit,
        max_participants: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        start_conference_on_create: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceCreateResponse:
        """
        Create a conference from an existing call leg using a `call_control_id` and a
        conference name. Upon creating the conference, the call will be automatically
        bridged to the conference. Conferences will expire after all participants have
        left the conference or after 4 hours regardless of the number of active
        participants.

        **Expected Webhooks:**

        - `conference.created`
        - `conference.participant.joined`
        - `conference.participant.left`
        - `conference.ended`
        - `conference.recording.saved`
        - `conference.floor.changed`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          name: Name of the conference

          beep_enabled: Whether a beep sound should be played when participants join and/or leave the
              conference.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string. The client_state will be updated for the creator call
              leg and will be used for all webhooks related to the created conference.

          comfort_noise: Toggle background comfort noise.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          duration_minutes: Time length (minutes) after which the conference will end.

          hold_audio_url: The URL of a file to be played to participants joining the conference. The URL
              can point to either a WAV or MP3 file. hold_media_name and hold_audio_url cannot
              be used together in one request. Takes effect only when
              "start_conference_on_create" is set to "false".

          hold_media_name: The media_name of a file to be played to participants joining the conference.
              The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file. Takes effect only when "start_conference_on_create" is set to
              "false".

          max_participants: The maximum number of active conference participants to allow. Must be between 2
              and 800. Defaults to 250

          region: Sets the region where the conference data will be hosted. Defaults to the region
              defined in user's data locality settings (Europe or US).

          start_conference_on_create: Whether the conference should be started on creation. If the conference isn't
              started all participants that join are automatically put on hold. Defaults to
              "true".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/conferences",
            body=maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "name": name,
                    "beep_enabled": beep_enabled,
                    "client_state": client_state,
                    "comfort_noise": comfort_noise,
                    "command_id": command_id,
                    "duration_minutes": duration_minutes,
                    "hold_audio_url": hold_audio_url,
                    "hold_media_name": hold_media_name,
                    "max_participants": max_participants,
                    "region": region,
                    "start_conference_on_create": start_conference_on_create,
                },
                conference_create_params.ConferenceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveResponse:
        """
        Retrieve an existing conference

        Args:
          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/conferences/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"region": region}, conference_retrieve_params.ConferenceRetrieveParams),
            ),
            cast_to=ConferenceRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: conference_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[Conference]:
        """Lists conferences.

        Conferences are created on demand, and will expire after all
        participants have left the conference or after 4 hours regardless of the number
        of active participants. Conferences are listed in descending order by
        `expires_at`.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[application_name][contains], filter[outbound.outbound_voice_profile_id],
              filter[leg_id], filter[application_session_id], filter[connection_id],
              filter[product], filter[failed], filter[from], filter[to], filter[name],
              filter[type], filter[occurred_at][eq/gt/gte/lt/lte], filter[status]

          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/conferences",
            page=SyncDefaultFlatPagination[Conference],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "region": region,
                    },
                    conference_list_params.ConferenceListParams,
                ),
            ),
            model=Conference,
        )

    def list_participants(
        self,
        conference_id: str,
        *,
        filter: conference_list_participants_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ConferenceListParticipantsResponse]:
        """
        Lists conference participants

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[muted],
              filter[on_hold], filter[whispering]

          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conference_id:
            raise ValueError(f"Expected a non-empty value for `conference_id` but received {conference_id!r}")
        return self._get_api_list(
            f"/conferences/{conference_id}/participants",
            page=SyncDefaultFlatPagination[ConferenceListParticipantsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "region": region,
                    },
                    conference_list_participants_params.ConferenceListParticipantsParams,
                ),
            ),
            model=ConferenceListParticipantsResponse,
        )

    def retrieve_participant(
        self,
        participant_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveParticipantResponse:
        """
        Retrieve details of a specific conference participant by their ID or label.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._get(
            f"/conferences/{id}/participants/{participant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveParticipantResponse,
        )

    def update_participant(
        self,
        participant_id: str,
        *,
        id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        soft_end_conference_on_exit: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceUpdateParticipantResponse:
        """
        Update properties of a conference participant.

        Args:
          beep_enabled: Whether entry/exit beeps are enabled for this participant.

          end_conference_on_exit: Whether the conference should end when this participant exits.

          soft_end_conference_on_exit: Whether the conference should soft-end when this participant exits. A soft end
              will stop new participants from joining but allow existing participants to
              remain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._patch(
            f"/conferences/{id}/participants/{participant_id}",
            body=maybe_transform(
                {
                    "beep_enabled": beep_enabled,
                    "end_conference_on_exit": end_conference_on_exit,
                    "soft_end_conference_on_exit": soft_end_conference_on_exit,
                },
                conference_update_participant_params.ConferenceUpdateParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceUpdateParticipantResponse,
        )


class AsyncConferencesResource(AsyncAPIResource):
    """Conference command operations"""

    @cached_property
    def actions(self) -> AsyncActionsResource:
        """Conference command operations"""
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncConferencesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        call_control_id: str,
        name: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        client_state: str | Omit = omit,
        comfort_noise: bool | Omit = omit,
        command_id: str | Omit = omit,
        duration_minutes: int | Omit = omit,
        hold_audio_url: str | Omit = omit,
        hold_media_name: str | Omit = omit,
        max_participants: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        start_conference_on_create: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceCreateResponse:
        """
        Create a conference from an existing call leg using a `call_control_id` and a
        conference name. Upon creating the conference, the call will be automatically
        bridged to the conference. Conferences will expire after all participants have
        left the conference or after 4 hours regardless of the number of active
        participants.

        **Expected Webhooks:**

        - `conference.created`
        - `conference.participant.joined`
        - `conference.participant.left`
        - `conference.ended`
        - `conference.recording.saved`
        - `conference.floor.changed`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          name: Name of the conference

          beep_enabled: Whether a beep sound should be played when participants join and/or leave the
              conference.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string. The client_state will be updated for the creator call
              leg and will be used for all webhooks related to the created conference.

          comfort_noise: Toggle background comfort noise.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          duration_minutes: Time length (minutes) after which the conference will end.

          hold_audio_url: The URL of a file to be played to participants joining the conference. The URL
              can point to either a WAV or MP3 file. hold_media_name and hold_audio_url cannot
              be used together in one request. Takes effect only when
              "start_conference_on_create" is set to "false".

          hold_media_name: The media_name of a file to be played to participants joining the conference.
              The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file. Takes effect only when "start_conference_on_create" is set to
              "false".

          max_participants: The maximum number of active conference participants to allow. Must be between 2
              and 800. Defaults to 250

          region: Sets the region where the conference data will be hosted. Defaults to the region
              defined in user's data locality settings (Europe or US).

          start_conference_on_create: Whether the conference should be started on creation. If the conference isn't
              started all participants that join are automatically put on hold. Defaults to
              "true".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/conferences",
            body=await async_maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "name": name,
                    "beep_enabled": beep_enabled,
                    "client_state": client_state,
                    "comfort_noise": comfort_noise,
                    "command_id": command_id,
                    "duration_minutes": duration_minutes,
                    "hold_audio_url": hold_audio_url,
                    "hold_media_name": hold_media_name,
                    "max_participants": max_participants,
                    "region": region,
                    "start_conference_on_create": start_conference_on_create,
                },
                conference_create_params.ConferenceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveResponse:
        """
        Retrieve an existing conference

        Args:
          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/conferences/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"region": region}, conference_retrieve_params.ConferenceRetrieveParams
                ),
            ),
            cast_to=ConferenceRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: conference_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Conference, AsyncDefaultFlatPagination[Conference]]:
        """Lists conferences.

        Conferences are created on demand, and will expire after all
        participants have left the conference or after 4 hours regardless of the number
        of active participants. Conferences are listed in descending order by
        `expires_at`.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[application_name][contains], filter[outbound.outbound_voice_profile_id],
              filter[leg_id], filter[application_session_id], filter[connection_id],
              filter[product], filter[failed], filter[from], filter[to], filter[name],
              filter[type], filter[occurred_at][eq/gt/gte/lt/lte], filter[status]

          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/conferences",
            page=AsyncDefaultFlatPagination[Conference],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "region": region,
                    },
                    conference_list_params.ConferenceListParams,
                ),
            ),
            model=Conference,
        )

    def list_participants(
        self,
        conference_id: str,
        *,
        filter: conference_list_participants_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        ConferenceListParticipantsResponse, AsyncDefaultFlatPagination[ConferenceListParticipantsResponse]
    ]:
        """
        Lists conference participants

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[muted],
              filter[on_hold], filter[whispering]

          region: Region where the conference data is located

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conference_id:
            raise ValueError(f"Expected a non-empty value for `conference_id` but received {conference_id!r}")
        return self._get_api_list(
            f"/conferences/{conference_id}/participants",
            page=AsyncDefaultFlatPagination[ConferenceListParticipantsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "region": region,
                    },
                    conference_list_participants_params.ConferenceListParticipantsParams,
                ),
            ),
            model=ConferenceListParticipantsResponse,
        )

    async def retrieve_participant(
        self,
        participant_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveParticipantResponse:
        """
        Retrieve details of a specific conference participant by their ID or label.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._get(
            f"/conferences/{id}/participants/{participant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveParticipantResponse,
        )

    async def update_participant(
        self,
        participant_id: str,
        *,
        id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        soft_end_conference_on_exit: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceUpdateParticipantResponse:
        """
        Update properties of a conference participant.

        Args:
          beep_enabled: Whether entry/exit beeps are enabled for this participant.

          end_conference_on_exit: Whether the conference should end when this participant exits.

          soft_end_conference_on_exit: Whether the conference should soft-end when this participant exits. A soft end
              will stop new participants from joining but allow existing participants to
              remain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._patch(
            f"/conferences/{id}/participants/{participant_id}",
            body=await async_maybe_transform(
                {
                    "beep_enabled": beep_enabled,
                    "end_conference_on_exit": end_conference_on_exit,
                    "soft_end_conference_on_exit": soft_end_conference_on_exit,
                },
                conference_update_participant_params.ConferenceUpdateParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceUpdateParticipantResponse,
        )


class ConferencesResourceWithRawResponse:
    def __init__(self, conferences: ConferencesResource) -> None:
        self._conferences = conferences

        self.create = to_raw_response_wrapper(
            conferences.create,
        )
        self.retrieve = to_raw_response_wrapper(
            conferences.retrieve,
        )
        self.list = to_raw_response_wrapper(
            conferences.list,
        )
        self.list_participants = to_raw_response_wrapper(
            conferences.list_participants,
        )
        self.retrieve_participant = to_raw_response_wrapper(
            conferences.retrieve_participant,
        )
        self.update_participant = to_raw_response_wrapper(
            conferences.update_participant,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        """Conference command operations"""
        return ActionsResourceWithRawResponse(self._conferences.actions)


class AsyncConferencesResourceWithRawResponse:
    def __init__(self, conferences: AsyncConferencesResource) -> None:
        self._conferences = conferences

        self.create = async_to_raw_response_wrapper(
            conferences.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            conferences.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            conferences.list,
        )
        self.list_participants = async_to_raw_response_wrapper(
            conferences.list_participants,
        )
        self.retrieve_participant = async_to_raw_response_wrapper(
            conferences.retrieve_participant,
        )
        self.update_participant = async_to_raw_response_wrapper(
            conferences.update_participant,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        """Conference command operations"""
        return AsyncActionsResourceWithRawResponse(self._conferences.actions)


class ConferencesResourceWithStreamingResponse:
    def __init__(self, conferences: ConferencesResource) -> None:
        self._conferences = conferences

        self.create = to_streamed_response_wrapper(
            conferences.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            conferences.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            conferences.list,
        )
        self.list_participants = to_streamed_response_wrapper(
            conferences.list_participants,
        )
        self.retrieve_participant = to_streamed_response_wrapper(
            conferences.retrieve_participant,
        )
        self.update_participant = to_streamed_response_wrapper(
            conferences.update_participant,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        """Conference command operations"""
        return ActionsResourceWithStreamingResponse(self._conferences.actions)


class AsyncConferencesResourceWithStreamingResponse:
    def __init__(self, conferences: AsyncConferencesResource) -> None:
        self._conferences = conferences

        self.create = async_to_streamed_response_wrapper(
            conferences.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            conferences.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            conferences.list,
        )
        self.list_participants = async_to_streamed_response_wrapper(
            conferences.list_participants,
        )
        self.retrieve_participant = async_to_streamed_response_wrapper(
            conferences.retrieve_participant,
        )
        self.update_participant = async_to_streamed_response_wrapper(
            conferences.update_participant,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        """Conference command operations"""
        return AsyncActionsResourceWithStreamingResponse(self._conferences.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["NumberOrderCreateParams", "PhoneNumber"]


class NumberOrderCreateParams(TypedDict, total=False):
    billing_group_id: str
    """Identifies the billing group associated with the phone number."""

    connection_id: str
    """Identifies the connection associated with this phone number."""

    customer_reference: str
    """A customer reference string for customer look ups."""

    messaging_profile_id: str
    """Identifies the messaging profile associated with the phone number."""

    phone_numbers: Iterable[PhoneNumber]


class PhoneNumber(TypedDict, total=False):
    phone_number: Required[str]
    """e164_phone_number"""

    bundle_id: str
    """ID of bundle to associate the number to"""

    requirement_group_id: str
    """ID of requirement group to use to satisfy number requirements"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["CustomerServiceRecordListParams", "Filter", "FilterCreatedAt", "FilterPhoneNumber", "FilterStatus", "Sort"]


class CustomerServiceRecordListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number][eq], filter[phone_number][in][],
    filter[status][eq], filter[status][in][], filter[created_at][lt],
    filter[created_at][gt]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class FilterCreatedAt(TypedDict, total=False):
    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filters records to those created after a specific date."""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filters records to those created before a specific date."""


_FilterPhoneNumberReservedKeywords = TypedDict(
    "_FilterPhoneNumberReservedKeywords",
    {
        "in": SequenceNotStr[str],
    },
    total=False,
)


class FilterPhoneNumber(_FilterPhoneNumberReservedKeywords, total=False):
    eq: str
    """Filters records to those with a specified number."""


_FilterStatusReservedKeywords = TypedDict(
    "_FilterStatusReservedKeywords",
    {
        "in": List[Literal["pending", "completed", "failed"]],
    },
    total=False,
)


class FilterStatus(_FilterStatusReservedKeywords, total=False):
    eq: Literal["pending", "completed", "failed"]
    """Filters records to those with a specific status."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number][eq], filter[phone_number][in][], filter[status][eq], filter[status][in][], filter[created_at][lt], filter[created_at][gt]
    """

    created_at: FilterCreatedAt

    phone_number: FilterPhoneNumber

    status: FilterStatus


class Sort(TypedDict, total=False):
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""

    value: Literal["created_at", "-created_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ..types import whatsapp_message_template_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.whatsapp_message_template_update_response import WhatsappMessageTemplateUpdateResponse
from ..types.whatsapp_message_template_retrieve_response import WhatsappMessageTemplateRetrieveResponse

__all__ = ["WhatsappMessageTemplatesResource", "AsyncWhatsappMessageTemplatesResource"]


class WhatsappMessageTemplatesResource(SyncAPIResource):
    """Manage Whatsapp message templates"""

    @cached_property
    def with_raw_response(self) -> WhatsappMessageTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WhatsappMessageTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WhatsappMessageTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WhatsappMessageTemplatesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappMessageTemplateRetrieveResponse:
        """
        Get a Whatsapp message template by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v2/whatsapp_message_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappMessageTemplateRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"] | Omit = omit,
        components: Iterable[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappMessageTemplateUpdateResponse:
        """
        Update a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/v2/whatsapp_message_templates/{id}",
            body=maybe_transform(
                {
                    "category": category,
                    "components": components,
                },
                whatsapp_message_template_update_params.WhatsappMessageTemplateUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappMessageTemplateUpdateResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v2/whatsapp_message_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncWhatsappMessageTemplatesResource(AsyncAPIResource):
    """Manage Whatsapp message templates"""

    @cached_property
    def with_raw_response(self) -> AsyncWhatsappMessageTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWhatsappMessageTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWhatsappMessageTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWhatsappMessageTemplatesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappMessageTemplateRetrieveResponse:
        """
        Get a Whatsapp message template by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v2/whatsapp_message_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappMessageTemplateRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"] | Omit = omit,
        components: Iterable[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappMessageTemplateUpdateResponse:
        """
        Update a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/v2/whatsapp_message_templates/{id}",
            body=await async_maybe_transform(
                {
                    "category": category,
                    "components": components,
                },
                whatsapp_message_template_update_params.WhatsappMessageTemplateUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappMessageTemplateUpdateResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v2/whatsapp_message_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class WhatsappMessageTemplatesResourceWithRawResponse:
    def __init__(self, whatsapp_message_templates: WhatsappMessageTemplatesResource) -> None:
        self._whatsapp_message_templates = whatsapp_message_templates

        self.retrieve = to_raw_response_wrapper(
            whatsapp_message_templates.retrieve,
        )
        self.update = to_raw_response_wrapper(
            whatsapp_message_templates.update,
        )
        self.delete = to_raw_response_wrapper(
            whatsapp_message_templates.delete,
        )


class AsyncWhatsappMessageTemplatesResourceWithRawResponse:
    def __init__(self, whatsapp_message_templates: AsyncWhatsappMessageTemplatesResource) -> None:
        self._whatsapp_message_templates = whatsapp_message_templates

        self.retrieve = async_to_raw_response_wrapper(
            whatsapp_message_templates.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            whatsapp_message_templates.update,
        )
        self.delete = async_to_raw_response_wrapper(
            whatsapp_message_templates.delete,
        )


class WhatsappMessageTemplatesResourceWithStreamingResponse:
    def __init__(self, whatsapp_message_templates: WhatsappMessageTemplatesResource) -> None:
        self._whatsapp_message_templates = whatsapp_message_templates

        self.retrieve = to_streamed_response_wrapper(
            whatsapp_message_templates.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            whatsapp_message_templates.update,
        )
        self.delete = to_streamed_response_wrapper(
            whatsapp_message_templates.delete,
        )


class AsyncWhatsappMessageTemplatesResourceWithStreamingResponse:
    def __init__(self, whatsapp_message_templates: AsyncWhatsappMessageTemplatesResource) -> None:
        self._whatsapp_message_templates = whatsapp_message_templates

        self.retrieve = async_to_streamed_response_wrapper(
            whatsapp_message_templates.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            whatsapp_message_templates.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            whatsapp_message_templates.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, TypedDict

__all__ = ["MissionUpdateMissionParams"]


class MissionUpdateMissionParams(TypedDict, total=False):
    description: str

    execution_mode: Literal["external", "managed"]

    instructions: str

    metadata: Dict[str, object]

    model: str

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SimCardGroupAction", "Settings"]


class Settings(BaseModel):
    """A JSON object representation of the action params."""

    private_wireless_gateway_id: Optional[str] = None
    """The identification of the related Private Wireless Gateway resource."""


class SimCardGroupAction(BaseModel):
    """This object represents a SIM card group action request.

    It allows tracking the current status of an operation that impacts the SIM card group and SIM card in it.
    """

    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    settings: Optional[Settings] = None
    """A JSON object representation of the action params."""

    sim_card_group_id: Optional[str] = None
    """The SIM card group identification."""

    status: Optional[Literal["in-progress", "completed", "failed"]] = None

    type: Optional[
        Literal[
            "set_private_wireless_gateway",
            "remove_private_wireless_gateway",
            "set_wireless_blocklist",
            "remove_wireless_blocklist",
        ]
    ] = None
    """Represents the type of the operation requested."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VoiceDesignRetrieveParams"]


class VoiceDesignRetrieveParams(TypedDict, total=False):
    version: int
    """Specific version number to retrieve. Defaults to the latest version."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .purchase import (
    PurchaseResource,
    AsyncPurchaseResource,
    PurchaseResourceWithRawResponse,
    AsyncPurchaseResourceWithRawResponse,
    PurchaseResourceWithStreamingResponse,
    AsyncPurchaseResourceWithStreamingResponse,
)
from .register import (
    RegisterResource,
    AsyncRegisterResource,
    RegisterResourceWithRawResponse,
    AsyncRegisterResourceWithRawResponse,
    RegisterResourceWithStreamingResponse,
    AsyncRegisterResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def purchase(self) -> PurchaseResource:
        """SIM Cards operations"""
        return PurchaseResource(self._client)

    @cached_property
    def register(self) -> RegisterResource:
        """SIM Cards operations"""
        return RegisterResource(self._client)

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def purchase(self) -> AsyncPurchaseResource:
        """SIM Cards operations"""
        return AsyncPurchaseResource(self._client)

    @cached_property
    def register(self) -> AsyncRegisterResource:
        """SIM Cards operations"""
        return AsyncRegisterResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

    @cached_property
    def purchase(self) -> PurchaseResourceWithRawResponse:
        """SIM Cards operations"""
        return PurchaseResourceWithRawResponse(self._actions.purchase)

    @cached_property
    def register(self) -> RegisterResourceWithRawResponse:
        """SIM Cards operations"""
        return RegisterResourceWithRawResponse(self._actions.register)


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

    @cached_property
    def purchase(self) -> AsyncPurchaseResourceWithRawResponse:
        """SIM Cards operations"""
        return AsyncPurchaseResourceWithRawResponse(self._actions.purchase)

    @cached_property
    def register(self) -> AsyncRegisterResourceWithRawResponse:
        """SIM Cards operations"""
        return AsyncRegisterResourceWithRawResponse(self._actions.register)


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

    @cached_property
    def purchase(self) -> PurchaseResourceWithStreamingResponse:
        """SIM Cards operations"""
        return PurchaseResourceWithStreamingResponse(self._actions.purchase)

    @cached_property
    def register(self) -> RegisterResourceWithStreamingResponse:
        """SIM Cards operations"""
        return RegisterResourceWithStreamingResponse(self._actions.register)


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

    @cached_property
    def purchase(self) -> AsyncPurchaseResourceWithStreamingResponse:
        """SIM Cards operations"""
        return AsyncPurchaseResourceWithStreamingResponse(self._actions.purchase)

    @cached_property
    def register(self) -> AsyncRegisterResourceWithStreamingResponse:
        """SIM Cards operations"""
        return AsyncRegisterResourceWithStreamingResponse(self._actions.register)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VoiceCloneUpdateParams"]


class VoiceCloneUpdateParams(TypedDict, total=False):
    name: Required[str]
    """New name for the voice clone."""

    gender: Literal["male", "female", "neutral"]
    """Updated gender for the voice clone."""

    language: str
    """Updated ISO 639-1 language code or `auto`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VoiceDesignData"]


class VoiceDesignData(BaseModel):
    """A voice design object with full version detail."""

    id: Optional[str] = None
    """Unique identifier for the voice design."""

    created_at: Optional[datetime] = None
    """Timestamp when the voice design was first created."""

    name: Optional[str] = None
    """Name of the voice design."""

    prompt: Optional[str] = None
    """Natural language prompt used to define the voice style for this version."""

    record_type: Optional[Literal["voice_design"]] = None
    """Identifies the resource type."""

    text: Optional[str] = None
    """Sample text used to synthesize this version."""

    updated_at: Optional[datetime] = None
    """Timestamp when the voice design was last updated."""

    version: Optional[int] = None
    """Version number of this voice design."""

    version_created_at: Optional[datetime] = None
    """Timestamp when this specific version was created."""

    voice_sample_size: Optional[int] = None
    """Size of the voice sample audio in bytes."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.portouts import event_list_params
from ...types.portouts.event_list_response import EventListResponse
from ...types.portouts.event_retrieve_response import EventRetrieveResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventRetrieveResponse:
        """
        Show a specific port-out event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/portouts/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: event_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[EventListResponse]:
        """
        Returns a list of all port-out events.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[event_type], filter[portout_id], filter[created_at]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts/events",
            page=SyncDefaultFlatPagination[EventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=cast(Any, EventListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    def republish(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Republish a specific port-out event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/portouts/events/{id}/republish",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncEventsResource(AsyncAPIResource):
    """Number portout operations"""

    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventRetrieveResponse:
        """
        Show a specific port-out event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/portouts/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: event_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EventListResponse, AsyncDefaultFlatPagination[EventListResponse]]:
        """
        Returns a list of all port-out events.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[event_type], filter[portout_id], filter[created_at]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts/events",
            page=AsyncDefaultFlatPagination[EventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=cast(Any, EventListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    async def republish(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Republish a specific port-out event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/portouts/events/{id}/republish",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = to_raw_response_wrapper(
            events.list,
        )
        self.republish = to_raw_response_wrapper(
            events.republish,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            events.list,
        )
        self.republish = async_to_raw_response_wrapper(
            events.republish,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            events.list,
        )
        self.republish = to_streamed_response_wrapper(
            events.republish,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            events.list,
        )
        self.republish = async_to_streamed_response_wrapper(
            events.republish,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["DocumentGenerateDownloadLinkResponse", "Data"]


class Data(BaseModel):
    url: str
    """Pre-signed temporary URL for downloading the document"""


class DocumentGenerateDownloadLinkResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .port_out_supporting_document import PortOutSupportingDocument

__all__ = ["SupportingDocumentListResponse"]


class SupportingDocumentListResponse(BaseModel):
    data: Optional[List[PortOutSupportingDocument]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["HTTP", "Request", "Response"]


class Request(BaseModel):
    """Request details."""

    headers: Optional[List[List[str]]] = None
    """List of headers, limited to 10kB."""

    url: Optional[str] = None


class Response(BaseModel):
    """Response details, optional."""

    body: Optional[str] = None
    """Raw response body, limited to 10kB."""

    headers: Optional[List[List[str]]] = None
    """List of headers, limited to 10kB."""

    status: Optional[int] = None


class HTTP(BaseModel):
    """HTTP request and response information."""

    request: Optional[Request] = None
    """Request details."""

    response: Optional[Response] = None
    """Response details, optional."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import bulk_sim_card_action_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.bulk_sim_card_action_list_response import BulkSimCardActionListResponse
from ..types.bulk_sim_card_action_retrieve_response import BulkSimCardActionRetrieveResponse

__all__ = ["BulkSimCardActionsResource", "AsyncBulkSimCardActionsResource"]


class BulkSimCardActionsResource(SyncAPIResource):
    """
    View SIM card actions, their progress and timestamps using the SIM Card Actions API
    """

    @cached_property
    def with_raw_response(self) -> BulkSimCardActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BulkSimCardActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BulkSimCardActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BulkSimCardActionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkSimCardActionRetrieveResponse:
        """This API fetches information about a bulk SIM card action.

        A bulk SIM card
        action contains details about a collection of individual SIM card actions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/bulk_sim_card_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkSimCardActionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_action_type: Literal["bulk_set_public_ips"] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[BulkSimCardActionListResponse]:
        """This API lists a paginated collection of bulk SIM card actions.

        A bulk SIM card
        action contains details about a collection of individual SIM card actions.

        Args:
          filter_action_type: Filter by action type.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/bulk_sim_card_actions",
            page=SyncDefaultFlatPagination[BulkSimCardActionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_action_type": filter_action_type,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    bulk_sim_card_action_list_params.BulkSimCardActionListParams,
                ),
            ),
            model=BulkSimCardActionListResponse,
        )


class AsyncBulkSimCardActionsResource(AsyncAPIResource):
    """
    View SIM card actions, their progress and timestamps using the SIM Card Actions API
    """

    @cached_property
    def with_raw_response(self) -> AsyncBulkSimCardActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBulkSimCardActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBulkSimCardActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBulkSimCardActionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkSimCardActionRetrieveResponse:
        """This API fetches information about a bulk SIM card action.

        A bulk SIM card
        action contains details about a collection of individual SIM card actions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/bulk_sim_card_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkSimCardActionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_action_type: Literal["bulk_set_public_ips"] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BulkSimCardActionListResponse, AsyncDefaultFlatPagination[BulkSimCardActionListResponse]]:
        """This API lists a paginated collection of bulk SIM card actions.

        A bulk SIM card
        action contains details about a collection of individual SIM card actions.

        Args:
          filter_action_type: Filter by action type.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/bulk_sim_card_actions",
            page=AsyncDefaultFlatPagination[BulkSimCardActionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_action_type": filter_action_type,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    bulk_sim_card_action_list_params.BulkSimCardActionListParams,
                ),
            ),
            model=BulkSimCardActionListResponse,
        )


class BulkSimCardActionsResourceWithRawResponse:
    def __init__(self, bulk_sim_card_actions: BulkSimCardActionsResource) -> None:
        self._bulk_sim_card_actions = bulk_sim_card_actions

        self.retrieve = to_raw_response_wrapper(
            bulk_sim_card_actions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            bulk_sim_card_actions.list,
        )


class AsyncBulkSimCardActionsResourceWithRawResponse:
    def __init__(self, bulk_sim_card_actions: AsyncBulkSimCardActionsResource) -> None:
        self._bulk_sim_card_actions = bulk_sim_card_actions

        self.retrieve = async_to_raw_response_wrapper(
            bulk_sim_card_actions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            bulk_sim_card_actions.list,
        )


class BulkSimCardActionsResourceWithStreamingResponse:
    def __init__(self, bulk_sim_card_actions: BulkSimCardActionsResource) -> None:
        self._bulk_sim_card_actions = bulk_sim_card_actions

        self.retrieve = to_streamed_response_wrapper(
            bulk_sim_card_actions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            bulk_sim_card_actions.list,
        )


class AsyncBulkSimCardActionsResourceWithStreamingResponse:
    def __init__(self, bulk_sim_card_actions: AsyncBulkSimCardActionsResource) -> None:
        self._bulk_sim_card_actions = bulk_sim_card_actions

        self.retrieve = async_to_streamed_response_wrapper(
            bulk_sim_card_actions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            bulk_sim_card_actions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_left_queue import CallLeftQueue

__all__ = ["CallLeftQueueWebhookEvent"]


class CallLeftQueueWebhookEvent(BaseModel):
    data: Optional[CallLeftQueue] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WireguardPeerCreateParams"]


class WireguardPeerCreateParams(TypedDict, total=False):
    wireguard_interface_id: Required[str]
    """The id of the wireguard interface associated with the peer."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .bulk_messaging_settings_update_phone_numbers import BulkMessagingSettingsUpdatePhoneNumbers

__all__ = ["MessagingNumbersBulkUpdateCreateResponse"]


class MessagingNumbersBulkUpdateCreateResponse(BaseModel):
    data: Optional[BulkMessagingSettingsUpdatePhoneNumbers] = None
